Stan Math Library: stan/math/prim/scal/prob/poisson_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

poisson_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_POISSON_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_POISSON_LOG_HPP
3
4 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
5 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_less.hpp>
9 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
10 #include <stan/math/prim/scal/err/check_not_nan.hpp>
11 #include <stan/math/prim/scal/fun/constants.hpp>
12 #include <stan/math/prim/scal/fun/multiply_log.hpp>
13 #include <stan/math/prim/scal/fun/gamma_q.hpp>
14 #include <stan/math/prim/scal/fun/value_of.hpp>
15 #include <stan/math/prim/scal/meta/include_summand.hpp>
16 #include <stan/math/prim/scal/meta/VectorView.hpp>
17 #include <boost/math/special_functions/fpclassify.hpp>
18 #include <boost/random/poisson_distribution.hpp>
19 #include <boost/random/variate_generator.hpp>
20 #include <limits>
21
22 namespace stan {
23
24 namespace math {
25
26 // Poisson(n|lambda) [lambda > 0; n >= 0]
27 template <bool propto, typename T_n, typename T_rate>
28 typename return_type<T_rate>::type
29 poisson_log(const T_n& n, const T_rate& lambda) {
30 typedef typename stan::partials_return_type<T_n, T_rate>::type
31 T_partials_return;
32
33 static const char* function("stan::math::poisson_log");
34
35 using boost::math::lgamma;
36 using stan::math::check_consistent_sizes;
37 using stan::math::check_not_nan;
38 using stan::math::check_nonnegative;
39 using stan::math::include_summand;
40 using stan::math::value_of;
41
42 // check if any vectors are zero length
43 if (!(stan::length(n)
44 && stan::length(lambda)))
45 return 0.0;
46
47 // set up return value accumulator
48 T_partials_return logp(0.0);
49
50 // validate args
51 check_nonnegative(function, "Random variable", n);
52 check_not_nan(function, "Rate parameter", lambda);
53 check_nonnegative(function, "Rate parameter", lambda);
54 check_consistent_sizes(function,
55 "Random variable", n,
56 "Rate parameter", lambda);
57
58 // check if no variables are involved and prop-to
59 if (!include_summand<propto, T_rate>::value)
60 return 0.0;
61
62 // set up expression templates wrapping scalars/vecs into vector views
63 VectorView<const T_n> n_vec(n);
64 VectorView<const T_rate> lambda_vec(lambda);
65 size_t size = max_size(n, lambda);
66
67 for (size_t i = 0; i < size; i++)
68 if (boost::math::isinf(lambda_vec[i]))
69 return LOG_ZERO;
70 for (size_t i = 0; i < size; i++)
71 if (lambda_vec[i] == 0 && n_vec[i] != 0)
72 return LOG_ZERO;
73
74 // return accumulator with gradients
75 OperandsAndPartials<T_rate> operands_and_partials(lambda);
76
77 using stan::math::multiply_log;
78 for (size_t i = 0; i < size; i++) {
79 if (!(lambda_vec[i] == 0 && n_vec[i] == 0)) {
80 if (include_summand<propto>::value)
81 logp -= lgamma(n_vec[i] + 1.0);
82 if (include_summand<propto, T_rate>::value)
83 logp += multiply_log(n_vec[i], value_of(lambda_vec[i]))
84 - value_of(lambda_vec[i]);
85 }
86
87 // gradients
88 if (!is_constant_struct<T_rate>::value)
89 operands_and_partials.d_x1[i]
90 += n_vec[i] / value_of(lambda_vec[i]) - 1.0;
91 }
92
93
94 return operands_and_partials.value(logp);
95 }
96
97 template <typename T_n,
98 typename T_rate>
99 inline
100 typename return_type<T_rate>::type
101 poisson_log(const T_n& n, const T_rate& lambda) {
102 return poisson_log<false>(n, lambda);
103 }
104 }
105 }
106 #endif
check_not_nan.hpp
stan::math::lgamma
fvar< T > lgamma(const fvar< T > &x)
Definition: lgamma.hpp:15
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::LOG_ZERO
const double LOG_ZERO
Definition: constants.hpp:175
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
gamma_q.hpp
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
multiply_log.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
boost::math::isinf
bool isinf(const stan::math::var &v)
Checks if the given number is infinite.
Definition: boost_isinf.hpp:22
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
VectorView.hpp
stan::math::poisson_log
return_type< T_rate >::type poisson_log(const T_n &n, const T_rate &lambda)
Definition: poisson_log.hpp:29
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
partials_return_type.hpp
OperandsAndPartials.hpp
stan::math::multiply_log
fvar< T > multiply_log(const fvar< T > &x1, const fvar< T > &x2)
Definition: multiply_log.hpp:15
constants.hpp
stan::math::size
int size(const std::vector< T > &x)
Return the size of the specified standard vector.
Definition: size.hpp:17
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
stan::math::check_nonnegative
bool check_nonnegative(const char *function, const char *name, const T_y &y)
Return true if y is non-negative.
Definition: check_nonnegative.hpp:66
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_less.hpp
check_consistent_sizes.hpp
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.